/////////////////////////////////////////////////////////////////////////////
// Name:        textdlg.h
// Purpose:     wxTextEntryDialog class
// Author:      Julian Smart
// Modified by:
// Created:     01/02/97
// RCS-ID:      $Id: textdlg.h,v 1.8 2004/05/23 20:50:25 JS Exp $
// Copyright:   (c) Julian Smart
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_TEXTDLG_H_BASE_
#define _WX_TEXTDLG_H_BASE_

#include "wx/generic/textdlgg.h"

#if WXWIN_COMPATIBILITY_2_4
    // for wxGetNumberFromUser()
    #include "wx/numdlg.h"
#endif // WXWIN_COMPATIBILITY_2_4

#endif // _WX_TEXTDLG_H_BASE_

