/*****************************************************************************
 * dr.h
 * (c)2001-2002 VideoLAN
 * $Id: dr.h 93 2004-10-19 19:17:49Z massiot $
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************************/

/*!
 * \file <dr.h>
 * \author Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 * \brief Gather all dr_*.h into one.
 *
 * Gathers all dr_*.h into one. Use this header if you need a lot of them.
 */

#ifndef _DVBPSI_DR_H_
#define _DVBPSI_DR_H_

#include "dr_02.h"
#include "dr_03.h"
#include "dr_04.h"
#include "dr_05.h"
#include "dr_06.h"
#include "dr_07.h"
#include "dr_08.h"
#include "dr_09.h"
#include "dr_0a.h"
#include "dr_0b.h"
#include "dr_0c.h"
#include "dr_0d.h"
#include "dr_0e.h"
#include "dr_0f.h"
#include "dr_42.h"
#include "dr_47.h"
#include "dr_48.h"
#include "dr_55.h"
#include "dr_59.h"

#else
#error "Multiple inclusions of dr.h"
#endif

