<!DOCTYPE kpartgui>
<kpartgui name="kvlc" version="0.1">

<ActionProperties>
  <Action name="open_disk" icon="cdrom_unmount"/>
  <Action name="open_stream" icon="network"/>
  <Action name="backward" icon="back"/>
  <Action name="stop" icon="stop"/>
  <Action name="play" icon="forward"/>
  <Action name="pause"	icon = "bottom"/>
  <Action name="slow" icon="undo"/>
  <Action name="fast" icon="redo"/>
  <Action name="prev" icon="start"/>
  <Action name="next" icon="finish"/>
</ActionProperties>

<MenuBar>
  <Menu name="file">
    <text>&amp;File</text>
    <Action name="open_disk"/>
    <Action name="open_stream"/>
  </Menu>
  <Menu name="view">
    <text>&amp;View</text>
    <Action name="program"/>
    <Action name="title"/>
    <Action name="chapter"/>
    <Separator lineSeparator="true"/>
    <Action name="view_messages"/>
    <Action name="view_stream_info"/>
  </Menu>
  <Menu name="settings">
    <text>&amp;Settings</text>
    <Action name="language"/>
    <Action name="subtitles"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" iconText="icononly" iconSize="16" noMerge="1">
  <Action name="file_open"/>
  <Action name="open_disk"/>
  <Action name="open_stream"/>
  <Separator lineSeparator="true"/>
  <Action name="backward"/>
  <Action name="stop"/>
  <Action name="play"/>
  <Action name="pause"/>
  <Action name="slow"/>
  <Action name="fast"/>
  <Separator lineSeparator="true"/>
  <Action name="prev"/>
  <Action name="next"/>
</ToolBar> 

</kpartgui>
